# Generated by roxygen2: do not edit by hand

S3method(print,summary.CovariateData)
export(aggregateCovariates)
export(computeStandardizedDifference)
export(convertPrespecSettingsToDetailedSettings)
export(createAnalysisDetails)
export(createCohortAttrCovariateSettings)
export(createCohortBasedCovariateSettings)
export(createCohortBasedTemporalCovariateSettings)
export(createCovariateSettings)
export(createDefaultCovariateSettings)
export(createDefaultTemporalCovariateSettings)
export(createDetailedCovariateSettings)
export(createDetailedTemporalCovariateSettings)
export(createEmptyCovariateData)
export(createTable1)
export(createTable1CovariateSettings)
export(createTemporalCovariateSettings)
export(createTemporalSequenceCovariateSettings)
export(filterByCohortDefinitionId)
export(filterByRowId)
export(getDbCohortAttrCovariatesData)
export(getDbCohortBasedCovariatesData)
export(getDbCovariateData)
export(getDbDefaultCovariateData)
export(getDefaultTable1Specifications)
export(isAggregatedCovariateData)
export(isCovariateData)
export(isTemporalCovariateData)
export(loadCovariateData)
export(saveCovariateData)
export(tidyCovariateData)
exportClasses(CovariateData)
exportMethods(show)
exportMethods(summary)
import(Andromeda)
import(DatabaseConnector)
import(dplyr)
importClassesFrom(DBI,DBIConnection)
importClassesFrom(DBI,DBIObject)
importClassesFrom(RSQLite,SQLiteConnection)
importFrom(SqlRender,loadRenderTranslateSql)
importFrom(SqlRender,render)
importFrom(SqlRender,translate)
importFrom(methods,is)
importFrom(rlang,.data)
importFrom(stats,aggregate)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(utils,read.csv)
